@@ -27,19 +27,23 @@ sql_escape_ident <- function(con, x) {
   UseMethod("sql_escape_ident")
 }
 #' @export
-sql_escape_ident.DBIConnection <- function(con, x) {
+sql_escape_ident.default <- function(con, x) {
   dbQuoteIdentifier(con, x)
 }
 #' @export
 sql_escape_ident.TestConnection <- function(con, x) {
-  sql_quote(x, "`")
+  if (methods::is(x, "SQL")) {
+    x
+  } else {
+    sql_quote(x, "`")
+  }
 }
 
 sql_escape_string <- function(con, x) {
   UseMethod("sql_escape_string")
 }
 #' @export
-sql_escape_string.DBIConnection <- function(con, x) {
+sql_escape_string.default <- function(con, x) {
   dbQuoteString(con, x)
 }
 #' @export
